GLOBAL.setmetatable(env, {__index=function(t, k)
	return GLOBAL.rawget(GLOBAL, k)
end})

local doors = {
	"wood",
	"stone",
	"organic",
	"iron",
	"pillar",
	"curtain",
	"round",
	"plate",
}

for k, v in pairs(doors) do
	AddPrefabPostInit(v.."_door", function(inst)
		inst:AddComponent("trans_door")
	end)
end

local is_door = {}
for k, v in pairs(doors) do
	is_door[v.."_door"] = true 
end

local old_use_door_fn = ACTIONS.USEDOOR.fn
ACTIONS.USEDOOR.fn = function(act)
	if act.target.components.door 
	and not act.target.components.door.disabled
	and act.target.components.trans_door then
		local door_id = act.target.components.door.target_door_id
		if door_id then
			local interior_spawner = GetInteriorSpawner()
			local target_door = interior_spawner:GetDoor(door_id)
			if not target_door then
				local interior_id = act.target.components.door.target_interior
				if interior_id then
					local target_interior = interior_spawner:GetInteriorByName(interior_id)
					if not target_interior then
						return false
					else
						-- the other door is save in data, not be spawned
						local no_check = true
						for k, v in pairs(target_interior.object_list) do
							if v.components.door and v.components.door.door_id == door_id then
								no_check = false
								break
							end
						end
						local prefabs = target_interior.prefabs or {}
						for k, v in pairs(prefabs) do
							if is_door[v.name] and v.my_door_id == door_id then
								no_check = false
								break
							end
						end
						if no_check then
							return false
						end
					end
				end
			end
		end
	end
	return old_use_door_fn(act)
end

local change_door = Action({}, 9)
change_door.id = "RESET_DOOR"
change_door.str = "RESET"
change_door.fn = function(act)
	if act.target.components.trans_door then
		act.target.components.trans_door:PushScreen()
		return true
	end
	return false
end
AddAction(change_door)

AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.RESET_DOOR, "give"))

AddComponentPostInit("interiorspawner", function(self)
	local old_fn = self.GetConnectedSurroundingPlayerRooms
	function self:GetConnectedSurroundingPlayerRooms(...)
		local found_doors = {}
		local doors = self:GetInteriorDoors(id)
		local curr_x, curr_y = self:GetPlayerRoomIndex(house_id, id)
		curr_x = curr_x or 0
		curr_y = curr_y or 0
		for _, door in ipairs(doors) do
			if door.inst.prefab ~= "prop_door" then
				local target_x, target_y = self:GetPlayerRoomIndex(house_id, door.target_interior)
				target_x = target_x or 0
				target_y = target_y or 0
				if target_y > curr_y and exclude_dir ~= "north" then -- North door
					found_doors["north"] = door.target_interior
				elseif target_y < curr_y and exclude_dir ~= "south" then -- South door
					found_doors["south"] = door.target_interior
				elseif target_x > curr_x and exclude_dir ~= "east" then -- East Door
					found_doors["east"] = door.target_interior
				elseif target_x < curr_x and exclude_dir ~= "west" then -- West Door
					found_doors["west"] = door.target_interior
				end
			end
		end
		return found_doors
	end
end)