local Screen = require "widgets/screen"
local Widget = require "widgets/widget"
local Menu = require "widgets/menu"
local TextEdit = require "widgets/textedit"
local Text = require "widgets/text"

local VALID_CHARS = [[ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_]]

local DoorResetScreen = Class(Screen, function(self, door, player)
	Screen._ctor(self, "DoorResetScreen")

	self:Hide()
	SetPause(true, "pause")

	self.door = door
	self.signer = player
	self.text = ""
	if self.door.components.door
	and self.door.components.door.target_door_id then
		self.text = self.door.components.door.target_id
	end

	self.root = self:AddChild(Widget("ROOT"))
	self.root:SetVAnchor(ANCHOR_MIDDLE)
	self.root:SetHAnchor(ANCHOR_MIDDLE)
	self.root:SetPosition(0, 0, 0)
	self.root:SetScaleMode(SCALEMODE_PROPORTIONAL)

	self.bg = self.root:AddChild(Image("images/globalpanels.xml", "panel.tex"))
	self.bg:SetPosition(0, 0, 0)
	self.bg:SetSize(500, 300)

	local sign_offset = 30
	local edit_height = 50
	local edit_width = 350
	local edit_bg_padding = 60
	local fontsize = 30

	self.sign_bg = self.root:AddChild(Image())
	self.sign_bg:SetTexture("images/ui.xml", "textbox_long.tex")
	self.sign_bg:SetPosition(0, sign_offset, 0)
	self.sign_bg:ScaleToSize(edit_width + edit_bg_padding, edit_height)

	self.sign_id = self.root:AddChild(Text(TITLEFONT, 30))
	self.sign_id:SetPosition(0, sign_offset+40, 0)
	if self.door.components.door then
		local txt = "My Id:"
		txt = txt..tostring(self.door.components.door.door_id)
		txt = txt.."\tTarget Id:"
		txt = txt..tostring(self.door.components.door.target_door_id)
		self.sign_id:SetString(txt)
	end

	self.signedit = self.root:AddChild(TextEdit(BODYTEXTFONT, fontsize, self.text or ""))
	self.signedit:SetPosition(0, sign_offset, 0)
	self.signedit:SetRegionSize(edit_width, edit_height)
	self.signedit:SetHAlign(ANCHOR_LEFT)
	self.signedit:SetFocusedImage(self.sign_bg, UI_ATLAS, "textbox_long_over.tex", "textbox_long.tex")
	self.signedit:SetTextLengthLimit(30)
	self.signedit:SetCharacterFilter(VALID_CHARS)
	self.signedit:SetAllowClipboardPaste(true)
	self.signedit.OnTextInput = function(_, text)
		local re = TextEdit.OnTextInput(_, text)
		return re
	end
	self.signedit.OnControl = function(_, control, down)
		return TextEdit.OnControl(_, control, down)
	end
	self.signedit.OnRawKey = function(_, key, down)
		if down then
			_.enter_focused = false
		else
			if key == KEY_BACKSPACE then
			elseif key == KEY_ENTER and _.enter_focused then
				self:OnWrite()
			end
		end
		return TextEdit.OnRawKey(_, key, down)
	end
	self.signedit.OnTextEntered = function(text)
		self.signedit.enter_focused = true
	end
	self.signedit.OnFocusMove = function(_, dir, down)
		if not _.dofocusmove or dir == MOVE_LEFT or dir == MOVE_RIGHT then
			return true
		end
		return Widget.OnFocusMove(_, dir, down)
	end

	self.menu = self.root:AddChild(Menu(nil, 200, true))
	self.menu:SetScale(0.6)
	self.menu:SetPosition(0, -50, 0)
	self.writebutton =
		self.menu:AddItem(
		"Edit",
		function()
			self:OnWrite()
		end
	)
	self.cancelbutton =
		self.menu:AddItem(
		"Cancel",
		function()
			self:OnCancel()
		end
	)
	self.menu:SetHRegPoint(ANCHOR_MIDDLE)

	self.signedit:SetFocusChangeDir(MOVE_DOWN, self.writebutton)
	self.writebutton:SetFocusChangeDir(MOVE_UP, self.signedit)
	self.cancelbutton:SetFocusChangeDir(MOVE_UP, self.signedit)

	self.default_focus = self.signedit
	self:Show()
	self.signedit:SetEditing(true)
	self.isopen = true
end)

function DoorResetScreen:OnWrite()
	if not self.isopen then
		return
	end

	local text = self.signedit:GetLineEditString()
	text = text == "" and nil or text
	if self.door.components.door then
		self.door.components.door.target_door_id = text
		local sub_text = nil
		if string.find(text, "_WEST") then
			sub_text = string.sub(text, 1, -6)
		elseif string.find(text, "_EAST") then
			sub_text = string.sub(text, 1, -6)
		elseif string.find(text, "_NORTH") then
			sub_text = string.sub(text, 1, -7)
		elseif string.find(text, "_SOUTH") then
			sub_text = string.sub(text, 1, -7)
		end
		if sub_text then
			if tonumber(sub_text) then
				sub_text = tonumber(sub_text)
			end
			self.door.components.door.target_interior = sub_text
		end
	end
	self:OnCancel()
end

function DoorResetScreen:OnCancel()
	if not self.isopen then
		return
	end

	TheFrontEnd:PopScreen(self)
	SetPause(false)
end

function DoorResetScreen:OnRawKey(key, down)
	if self._base.OnRawKey(self, key, down) then
		return true
	end
	if down then
		self.keyin = true
	else
		if key == KEY_ENTER and self.keyin then
			self:OnWrite()
			return true
		end
		self.keyin = false
	end
end

function DoorResetScreen:OnControl(control, down)
	if self._base.OnControl(self, control, down) then
		return true
	end
	if not down then
		if control == CONTROL_CANCEL then
			self:OnCancel()
			return true
		end
	end
end

return DoorResetScreen
